@@ -5,13 +5,10 @@
 package main
 
 import (
-	"bytes"
 	"encoding/json"
-	"fmt"
 	"log"
 	"os"
 	"path"
-	"path/filepath"
 	"strings"
 
 	"github.com/la5nta/pat/cfg"
@@ -34,89 +31,90 @@ func LoadConfig(cfgPath string, fallback cfg.Config) (config cfg.Config, err err
 		config.ConnectAliases["telnet"] = cfg.DefaultConfig.ConnectAliases["telnet"]
 	}
 
+	// TODO: Remove after some release cycles (2023-05-21)
+	// Rewrite deprecated serial-tnc:// aliases to ax25-serial-tnc://
+	var deprecatedAliases []string
+	for k, v := range config.ConnectAliases {
+		if !strings.HasPrefix(v, MethodSerialTNCDeprecated+"://") {
+			continue
+		}
+		deprecatedAliases = append(deprecatedAliases, k)
+		config.ConnectAliases[k] = strings.Replace(v, MethodSerialTNCDeprecated, MethodAX25SerialTNC, 1)
+	}
+	if len(deprecatedAliases) > 0 {
+		log.Printf("Alias(es) %s uses deprecated transport scheme %s://. Please use %s:// instead.", strings.Join(deprecatedAliases, ", "), MethodSerialTNCDeprecated, MethodAX25SerialTNC)
+	}
+
 	// Ensure ServiceCodes has a default value
 	if len(config.ServiceCodes) == 0 {
 		config.ServiceCodes = cfg.DefaultConfig.ServiceCodes
 	}
 
+	// Ensure we have a default AX.25 engine
+	if config.AX25.Engine == "" {
+		config.AX25.Engine = cfg.DefaultAX25Engine()
+	}
+
+	// Ensure we have a default AGWPE config
+	if config.AGWPE == (cfg.AGWPEConfig{}) {
+		config.AGWPE = cfg.DefaultConfig.AGWPE
+	}
+
+	// Ensure we have a default AX.25 Linux config
+	if config.AX25Linux == (cfg.AX25LinuxConfig{}) {
+		config.AX25Linux = cfg.DefaultConfig.AX25Linux
+	}
+	// TODO: Remove after some release cycles (2023-04-30)
+	if v := config.AX25.AXPort; v != "" && v != config.AX25Linux.Port {
+		log.Println("Using deprecated configuration option ax25.port. Please set ax25_linux.port instead.")
+		config.AX25Linux.Port = v
+	}
+
 	// Ensure Pactor has a default value
 	if config.Pactor == (cfg.PactorConfig{}) {
 		config.Pactor = cfg.DefaultConfig.Pactor
 	}
 
 	// Ensure VARA FM and VARA HF has default values
-	if config.VaraHF == (cfg.VaraConfig{}) {
+	if config.VaraHF.IsZero() {
 		config.VaraHF = cfg.DefaultConfig.VaraHF
 	}
-	if config.VaraFM == (cfg.VaraConfig{}) {
+	if config.VaraFM.IsZero() {
 		config.VaraFM = cfg.DefaultConfig.VaraFM
 	}
 
+	// Ensure GPSd has a default value
+	if config.GPSd == (cfg.GPSdConfig{}) {
+		config.GPSd = cfg.DefaultConfig.GPSd
+	}
 	// TODO: Remove after some release cycles (2019-09-29)
-	if config.GPSdAddrLegacy != "" {
-		config.GPSd.Addr = config.GPSdAddrLegacy
+	if v := config.GPSdAddrLegacy; v != "" && v != config.GPSd.Addr {
+		log.Println("Using deprecated configuration option gpsd_addr. Please set gpsd.addr instead.")
+		config.GPSd.Addr = v
 	}
 
-	// TODO: config FormsPath is deprecated in favor of --forms flag
-	homeDir, _ := os.UserHomeDir()
-	formsOldDefault := filepath.Join(homeDir, ".wl2k", "Standard_Forms")
-	// Ignore the old default config value
-	if config.FormsPath == formsOldDefault {
-		config.FormsPath = ""
+	// Ensure SerialTNC has a default hbaud and serialbaud
+	if config.SerialTNC.HBaud == 0 {
+		config.SerialTNC.HBaud = cfg.DefaultConfig.SerialTNC.HBaud
 	}
-	if config.FormsPath != "" {
-		log.Println("Using deprecated configuration option 'forms_path'. Please use --forms flag instead.")
-		// clean up FormsPath (normalizes trailing slashes, and embedded '.' )
-		config.FormsPath = filepath.Clean(config.FormsPath)
-		config.FormsPath = strings.ReplaceAll(config.FormsPath, "\\", "/")
+	if config.SerialTNC.SerialBaud == 0 {
+		config.SerialTNC.SerialBaud = cfg.DefaultConfig.SerialTNC.SerialBaud
 	}
-
 	// Compatibility for the old baudrate field for serial-tnc
-	if v := config.SerialTNC.BaudrateLegacy; v != 0 && config.SerialTNC.HBaud == 0 {
+	if v := config.SerialTNC.BaudrateLegacy; v != 0 && v != config.SerialTNC.HBaud {
+		// Since we changed the default value from 9600 to 1200, we can't warn about this without causing confusion.
 		debug.Printf("Legacy serial_tnc.baudrate config detected (%d). Translating to serial_tnc.hbaud.", v)
 		config.SerialTNC.HBaud = v
-		config.SerialTNC.BaudrateLegacy = 0
 	}
+
 	return config, nil
 }
 
-func replaceDeprecatedCMSHostname(path string, data []byte) ([]byte, error) {
-	const o = "@server.winlink.org:8772/wl2k"
-	const n = "@cms.winlink.org:8772/wl2k"
-
-	if !bytes.Contains(data, []byte(o)) {
-		return data, nil
-	}
-
-	data = bytes.ReplaceAll(data, []byte(o), []byte(n))
-
-	f, err := os.Open(path)
-	if err != nil {
-		return data, err
-	}
-	stat, err := f.Stat()
-	f.Close()
-	if err != nil {
-		return data, err
-	}
-	return data, os.WriteFile(path, data, stat.Mode())
-}
-
 func ReadConfig(path string) (config cfg.Config, err error) {
 	data, err := os.ReadFile(path)
 	if err != nil {
 		return
 	}
-
-	// TODO: Remove after some release cycles (2017-11-09)
-	data, err = replaceDeprecatedCMSHostname(path, data)
-	if err != nil {
-		fmt.Println("Failed to rewrite deprecated CMS hostname:", err)
-		fmt.Println("Please update your config's 'telnet' connect alias manually to:")
-		fmt.Println(cfg.DefaultConfig.ConnectAliases["telnet"])
-		fmt.Println("")
-	}
-
 	err = json.Unmarshal(data, &config)
 	return
 }
