@@ -8,26 +8,30 @@ import (
 	"context"
 	"errors"
 	"fmt"
-	"github.com/la5nta/pat/cfg"
-	"github.com/la5nta/pat/internal/debug"
 	"log"
 	"strconv"
 	"strings"
 	"time"
 
+	"github.com/la5nta/pat/cfg"
+	"github.com/la5nta/pat/internal/debug"
+
 	"github.com/harenber/ptc-go/v2/pactor"
 	"github.com/la5nta/wl2k-go/transport"
 	"github.com/la5nta/wl2k-go/transport/ardop"
+	"github.com/la5nta/wl2k-go/transport/ax25/agwpe"
 	"github.com/n8jja/Pat-Vara/vara"
 
-	// Register other dialers
+	// Register stateless dialers
 	_ "github.com/la5nta/wl2k-go/transport/ax25"
 	_ "github.com/la5nta/wl2k-go/transport/telnet"
 )
 
 var (
-	dialing     *transport.URL // The connect URL currently being dialed (if any)
+	dialing *transport.URL // The connect URL currently being dialed (if any)
+
 	adTNC       *ardop.TNC     // Pointer to the ARDOP TNC used by Listen and Connect
+	agwpeTNC    *agwpe.TNCPort // Pointer to the AGWPE TNC combined TNC and Port
 	pModem      *pactor.Modem
 	varaHFModem *vara.Modem
 	varaFMModem *vara.Modem
@@ -55,7 +59,9 @@ func Connect(connectStr string) (success bool) {
 	}
 
 	// Hack around bug in frontend which may occur if the status updates too quickly.
-	defer func() { time.Sleep(time.Second); websocketHub.UpdateStatus() }()
+	if websocketHub != nil {
+		defer func() { time.Sleep(time.Second); websocketHub.UpdateStatus() }()
+	}
 
 	debug.Printf("connectStr: %s", connectStr)
 	url, err := transport.ParseURL(connectStr)
@@ -64,8 +70,25 @@ func Connect(connectStr string) (success bool) {
 		return false
 	}
 
+	// TODO: Remove after some release cycles (2023-05-21)
+	// Rewrite legacy serial-tnc scheme.
+	if url.Scheme == MethodSerialTNCDeprecated {
+		log.Printf("Transport scheme %s:// is deprecated, use %s:// instead.", MethodSerialTNCDeprecated, MethodAX25SerialTNC)
+		url.Scheme = MethodAX25SerialTNC
+	}
+
+	// Rewrite the generic ax25:// scheme to use a specified AX.25 engine.
+	if url.Scheme == MethodAX25 {
+		url.Scheme = defaultAX25Method()
+	}
+
 	// Init TNCs
 	switch url.Scheme {
+	case MethodAX25AGWPE:
+		if err := initAGWPE(); err != nil {
+			log.Println(err)
+			return
+		}
 	case MethodArdop:
 		if err := initArdopTNC(); err != nil {
 			log.Println(err)
@@ -81,12 +104,12 @@ func Connect(connectStr string) (success bool) {
 			return
 		}
 	case MethodVaraHF:
-		if err := initVaraModem(varaHFModem, MethodVaraHF, config.VaraHF); err != nil {
+		if err := initVaraHFModem(); err != nil {
 			log.Println(err)
 			return
 		}
 	case MethodVaraFM:
-		if err := initVaraModem(varaFMModem, MethodVaraFM, config.VaraFM); err != nil {
+		if err := initVaraFMModem(); err != nil {
 			log.Println(err)
 			return
 		}
@@ -100,9 +123,9 @@ func Connect(connectStr string) (success bool) {
 	// Set default host interface address
 	if url.Host == "" {
 		switch url.Scheme {
-		case MethodAX25:
-			url.Host = config.AX25.Port
-		case MethodSerialTNC:
+		case MethodAX25Linux:
+			url.Host = config.AX25Linux.Port
+		case MethodAX25SerialTNC:
 			url.Host = config.SerialTNC.Path
 			if hbaud := config.SerialTNC.HBaud; hbaud > 0 {
 				url.Params.Set("hbaud", fmt.Sprint(hbaud))
@@ -124,13 +147,11 @@ func Connect(connectStr string) (success bool) {
 			return
 		}
 
-		switch url.Scheme {
-		case MethodAX25, MethodSerialTNC:
+		if strings.HasPrefix(url.Scheme, MethodAX25) {
 			log.Printf("Radio-Only is not available for %s", url.Scheme)
 			return
-		default:
-			url.SetUser(url.User.Username() + "-T")
 		}
+		url.SetUser(url.User.Username() + "-T")
 	}
 
 	// QSY
@@ -291,31 +312,96 @@ func initPactorModem(cmdlineinit string) error {
 	return nil
 }
 
-func initVaraModem(vModem *vara.Modem, scheme string, conf cfg.VaraConfig) error {
-	if vModem != nil {
-		_ = vModem.Close()
+func initVaraHFModem() error {
+	if varaHFModem != nil && varaHFModem.Ping() {
+		return nil
 	}
-	vConf := vara.ModemConfig{
-		Host:     conf.Host,
-		CmdPort:  conf.CmdPort,
-		DataPort: conf.DataPort,
-	}
-	var err error
-	vModem, err = vara.NewModem(scheme, fOptions.MyCall, vConf)
+	m, err := initVaraModem(MethodVaraHF, config.VaraHF)
 	if err != nil {
-		return fmt.Errorf("vara initialization failed: %w", err)
+		return err
 	}
+	if bw := config.VaraHF.Bandwidth; bw != 0 {
+		if err := m.SetBandwidth(fmt.Sprint(bw)); err != nil {
+			m.Close()
+			return err
+		}
+	}
+	varaHFModem = m
+	return nil
+}
 
-	transport.RegisterDialer(scheme, vModem)
+func initVaraFMModem() error {
+	if varaFMModem != nil && varaFMModem.Ping() {
+		return nil
+	}
+	m, err := initVaraModem(MethodVaraFM, config.VaraFM)
+	if err != nil {
+		return err
+	}
+	varaFMModem = m
+	return nil
+}
 
-	if !conf.PTTControl {
+func initVaraModem(scheme string, conf cfg.VaraConfig) (*vara.Modem, error) {
+	vConf := vara.ModemConfig{
+		Host:     conf.Host(),
+		CmdPort:  conf.CmdPort(),
+		DataPort: conf.DataPort(),
+	}
+	m, err := vara.NewModem(scheme, fOptions.MyCall, vConf)
+	if err != nil {
+		return nil, fmt.Errorf("vara initialization failed: %w", err)
+	}
+	transport.RegisterDialer(scheme, m)
+
+	if conf.PTTControl {
+		rig, ok := rigs[conf.Rig]
+		if !ok {
+			m.Close()
+			return nil, fmt.Errorf("unable to set PTT rig '%s': not defined or not loaded", conf.Rig)
+		}
+		m.SetPTT(rig)
+	}
+	v, _ := m.Version()
+	log.Printf("VARA modem (%s) initialized", v)
+	return m, nil
+}
+
+func initAGWPE() error {
+	if agwpeTNC != nil && agwpeTNC.Ping() == nil {
 		return nil
 	}
 
-	rig, ok := rigs[conf.Rig]
-	if !ok {
-		return fmt.Errorf("unable to set PTT rig '%s': not defined or not loaded", conf.Rig)
+	if agwpeTNC != nil {
+		agwpeTNC.Close()
 	}
-	vModem.SetPTT(rig)
+
+	var err error
+	agwpeTNC, err = agwpe.OpenPortTCP(config.AGWPE.Addr, config.AGWPE.RadioPort, fOptions.MyCall)
+	if err != nil {
+		return fmt.Errorf("AGWPE TNC initialization failed: %w", err)
+	}
+
+	if v, err := agwpeTNC.Version(); err != nil {
+		return fmt.Errorf("AGWPE TNC initialization failed: %w", err)
+	} else {
+		log.Printf("AGWPE TNC (%s) initialized", v)
+	}
+
+	transport.RegisterContextDialer(MethodAX25AGWPE, agwpeTNC)
 	return nil
 }
+
+// defaultAX25Method resolves the generic ax25:// scheme to a implementation specific scheme.
+func defaultAX25Method() string {
+	switch config.AX25.Engine {
+	case cfg.AX25EngineAGWPE:
+		return MethodAX25AGWPE
+	case cfg.AX25EngineSerialTNC:
+		return MethodAX25SerialTNC
+	case cfg.AX25EngineLinux:
+		return MethodAX25Linux
+	default:
+		panic(fmt.Sprintf("invalid ax25 engine: %s", config.AX25.Engine))
+	}
+}
