@@ -12,7 +12,6 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-
 	"html/template"
 	"io"
 	"io/fs"
@@ -458,32 +457,19 @@ func bandwidthsHandler(w http.ResponseWriter, req *http.Request) {
 	}
 	mode := strings.ToLower(req.FormValue("mode"))
 	resp := BandwidthResponse{Mode: mode, Bandwidths: []string{}}
-	switch mode {
-	case MethodPactor:
-		fallthrough
-	case MethodAX25:
-		fallthrough
-	case MethodTelnet:
-		fallthrough
-	case MethodVaraFM:
-		fallthrough
-	case MethodSerialTNC:
-		// bandwidth not supported, return empty list
-	case MethodArdop:
+	switch {
+	case mode == MethodArdop:
 		for _, bw := range ardop.Bandwidths() {
 			resp.Bandwidths = append(resp.Bandwidths, bw.String())
 		}
 		if bw := config.Ardop.ARQBandwidth; !bw.IsZero() {
 			resp.Default = bw.String()
 		}
-	case MethodVaraHF:
+	case mode == MethodVaraHF:
 		resp.Bandwidths = vara.Bandwidths()
 		if bw := config.VaraHF.Bandwidth; bw != 0 {
 			resp.Default = fmt.Sprintf("%d", bw)
 		}
-	default:
-		http.Error(w, "mode not recognized", http.StatusBadRequest)
-		return
 	}
 	_ = json.NewEncoder(w).Encode(resp)
 }
