@@ -35,13 +35,19 @@ import (
 )
 
 const (
-	MethodArdop     = "ardop"
-	MethodTelnet    = "telnet"
-	MethodAX25      = "ax25"
-	MethodSerialTNC = "serial-tnc"
-	MethodPactor    = "pactor"
-	MethodVaraHF    = "varahf"
-	MethodVaraFM    = "varafm"
+	MethodArdop  = "ardop"
+	MethodTelnet = "telnet"
+	MethodPactor = "pactor"
+	MethodVaraHF = "varahf"
+	MethodVaraFM = "varafm"
+
+	MethodAX25          = "ax25"
+	MethodAX25AGWPE     = MethodAX25 + "+agwpe"
+	MethodAX25Linux     = MethodAX25 + "+linux"
+	MethodAX25SerialTNC = MethodAX25 + "+serial-tnc"
+
+	// TODO: Remove after some release cycles (2023-05-21)
+	MethodSerialTNCDeprecated = "serial-tnc"
 )
 
 var commands = []Command{
@@ -301,7 +307,6 @@ func main() {
 	if err != nil {
 		log.Fatalf("Unable to load/write config: %s", err)
 	}
-	maybeUpdateFormsDir(os.Args)
 
 	// Initialize logger
 	f, err := os.Create(fOptions.LogPath)
@@ -384,20 +389,6 @@ func main() {
 	cmd.HandleFunc(ctx, args)
 }
 
-func maybeUpdateFormsDir(args []string) {
-	// Backward compatibility for config file forms_path
-	formsFlagExplicitlyUsed := false
-	for i := 0; i < len(args); i++ {
-		if strings.HasPrefix(args[i], "--forms") {
-			formsFlagExplicitlyUsed = true
-		}
-	}
-	if !formsFlagExplicitlyUsed && config.FormsPath != "" {
-		debug.Printf("Updating forms dir based on config file: %v", config.FormsPath)
-		fOptions.FormsPath = config.FormsPath
-	}
-}
-
 func configureHandle(ctx context.Context, args []string) {
 	// Ensure config file has been written
 	_, err := ReadConfig(fOptions.ConfigPath)
@@ -494,27 +485,32 @@ func cleanup() {
 	listenHub.Close()
 	if adTNC != nil {
 		if err := adTNC.Close(); err != nil {
-			log.Fatalf("Failure to close ardop TNC: %s", err)
+			log.Printf("Failure to close ardop TNC: %s", err)
 		}
 	}
 	if pModem != nil {
 		if err := pModem.Close(); err != nil {
-			log.Fatalf("Failure to close pactor modem: %s", err)
+			log.Printf("Failure to close pactor modem: %s", err)
 		}
 	}
 
 	if varaFMModem != nil {
 		if err := varaFMModem.Close(); err != nil {
-			log.Fatalf("Failure to close varafm modem: %s", err)
+			log.Printf("Failure to close varafm modem: %s", err)
 		}
 	}
 
 	if varaHFModem != nil {
 		if err := varaHFModem.Close(); err != nil {
-			log.Fatalf("Failure to close varahf modem: %s", err)
+			log.Printf("Failure to close varahf modem: %s", err)
 		}
 	}
 
+	if agwpeTNC != nil {
+		if err := agwpeTNC.Close(); err != nil {
+			log.Printf("Failure to close AGWPE TNC: %s", err)
+		}
+	}
 	eventLog.Close()
 }
 
